/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


package edu.louisville.physics.jcm.jabinit;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import java.awt.Dimension;


public class AtomsTable extends JTable {

	CrystalData crystalData;	

	AtomsTable() {
		super();
		setModel(new AtomsTableModel());
		getTableHeader().setReorderingAllowed(false);
	}
	
	public int getTableHeight() {
		return (getRowHeight()+getRowMargin())*getRowCount()+getRowMargin();
	}
	
	public void setCrystalData(CrystalData crystalData) {
		this.crystalData = crystalData;
		revalidate();
		repaint();
	}

	class AtomsTableModel extends AbstractTableModel {
        private String[] columnNames = {" ","1", "2", "3"};

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
			if (crystalData == null) return 2;
            return crystalData.nAtoms;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            if (crystalData == null) return null;
			if (columnNames[col] == " ") 
				return crystalData.getAtomName(row);
			if (columnNames[col] == "1")
				return new Float(crystalData.xred[row].x);
			if (columnNames[col] == "2")
				return new Float(crystalData.xred[row].y);
			if (columnNames[col] == "3")
				return new Float(crystalData.xred[row].z);
			return null;
        }

        public Class getColumnClass(int col) {
			if (getColumnName(col) == " ") return String.class;
            return Float.class;
        }

        public boolean isCellEditable(int row, int col) {
			if (getColumnName(col) == " ") return false;
			return true;
        }
		
        public void setValueAt(Object value, int row, int col) {
            if (crystalData == null) return;
			if (!(value instanceof Number)) return;
			if (columnNames[col] == "1")
				crystalData.xred[row].x = ((Float)value).floatValue();
			else if (columnNames[col] == "2")
				crystalData.xred[row].y = ((Float)value).floatValue();
			else if (columnNames[col] == "3")
				crystalData.xred[row].z = ((Float)value).floatValue();
            fireTableCellUpdated(row, col);
        }
	}
}
