/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import edu.berkeley.eecs.ptolemy.plot.Plot;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;

public class BandPlot extends Plot {

	public BandPlot() {
		super();
		setXRange(0,39);
		setYRange(-10,10);
		setLabelFont("Serif");
		setYLabel("Energy (eV)");
		addXTick("L", 0);
		addXTick("\u0393", 10);
		addXTick("X", 22);
		addXTick("\u0393", 39);
	//	setBackground(null);
	}

	public void clearData() {
		clear(false);
		clearLegends();
	}

	public void plotData(double[][] data) {
		clearData();

		double max = data[0][0];
		for (int i=0; i<data[0].length; i++) {
			for (int j=0; j<4; j++) {
				if (max < data[j][i]) max = data[j][i];
			}
		}
		
		for (int i=0; i<data[0].length; i++) {
			for (int j=0; j<data.length; j++) {
				addPoint(j,i,data[j][i]-max,true);
			}
		}


		//add labels to legend
//		for (int j=0; j<toShow.length; j++) {
//			addLegend(toShow[j],labels[toShow[j]]);
//		}

		fillPlot();		
		repaint();
	}

	protected synchronized void _drawPlot(Graphics graphics,
        boolean clearfirst, Rectangle drawRectangle) {
		Graphics2D g2 = (Graphics2D) graphics;

		//turn on antialiasing
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		super._drawPlot(g2, clearfirst, drawRectangle);
    }
}
	
