/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JToolBar;
import javax.swing.JTabbedPane;
import javax.swing.JRootPane;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Rectangle; 
import java.awt.Dimension;
import java.awt.event.KeyEvent;
import edu.louisville.physics.jcm.swing.MinimalScrollPane;
import edu.louisville.physics.jcm.utils.AppletImage;

public class CrystalControl extends JPanel {
	JButton runButton, resetButton;
	Icon blackRun, redRun;
	JComboBox crystalSelector;
	PrimitiveVectorTable primitiveVectorTable;
	AtomsTable atomsTable;
	PlotPointsTable plotPointsTable;
	MiscParametersTable miscParametersTable;
	MinimalScrollPane scrollPane;
	JTabbedPane tabbedPane;

	CrystalControl(JTabbedPane tabbedPane) {
		super();
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		this.tabbedPane = tabbedPane;
		
		//Primitive vector table
		add(new JLabel("primitive vectors:"));
		primitiveVectorTable = new PrimitiveVectorTable();
		scrollPane = new MinimalScrollPane(primitiveVectorTable);
		scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		add(scrollPane);
		add(Box.createVerticalGlue());

		//atoms table
		add(new JLabel("reduced atom coordinates:"));
		atomsTable = new AtomsTable();
		scrollPane = new MinimalScrollPane(atomsTable);
		scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		add(scrollPane);
		add(Box.createVerticalGlue());
		
		//band plot points table
		add(new JLabel("band plot points:"));
		plotPointsTable = new PlotPointsTable();
		scrollPane = new MinimalScrollPane(plotPointsTable);
		scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		add(scrollPane);
		add(Box.createVerticalGlue());		
		
		//other parameters
		add(new JLabel("other parameters:"));
		miscParametersTable = new MiscParametersTable();
		scrollPane = new MinimalScrollPane(miscParametersTable);
		scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		add(scrollPane);
		add(Box.createVerticalGlue());	
		
		//tool bar
		JToolBar toolbar = new JToolBar();
		toolbar.setAlignmentX(Component.LEFT_ALIGNMENT);
		toolbar.setFloatable(false);
		add(toolbar);
		
		//Element selector
		String[] crystalNames = {"Si", "Ge", "AlAs", "GaAs"};
		crystalSelector = new JComboBox(crystalNames);
		crystalSelector.setToolTipText("choose a crystal");
		crystalSelector.setAlignmentX(Component.LEFT_ALIGNMENT);
		crystalSelector.setPrototypeDisplayValue("GaAsXX");
		crystalSelector.setMaximumSize(crystalSelector.getMinimumSize());
		crystalSelector.setPreferredSize(crystalSelector.getMinimumSize());
		
		toolbar.add(crystalSelector);
		
		// reset data
		resetButton = new JButton(AppletImage.createAppletImageIcon(Jabinit.class, "images/reuseicon.gif","reset data"));
		resetButton.setToolTipText("reset crystal data");
		resetButton.setMargin(new Insets(2,2,2,2));
		resetButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		toolbar.add(resetButton);
		
		// run button
		blackRun = AppletImage.createAppletImageIcon(Jabinit.class, "images/runIcon.gif", "run");
		redRun = AppletImage.createAppletImageIcon(Jabinit.class, "images/redrunIcon.gif", "run");
		runButton = new JButton(redRun);
		runButton.setMnemonic(KeyEvent.VK_R);
		runButton.setToolTipText("run the calculation");
		runButton.setMargin(new Insets(2,2,2,2));
		runButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		toolbar.add(runButton);
	}
	
	//make top of side panel flush with the top of the tabbed pane
	public Dimension getMaximumSize() {
		Rectangle tb = tabbedPane.getUI().getTabBounds(tabbedPane,0);
		JRootPane rootPane = SwingUtilities.getRootPane(this);
		Insets ins = rootPane.getInsets();
		int h = rootPane.getSize().height - tb.height - tb.y - ins.top - ins.bottom;
		return new Dimension(250,h);
	}
	
	public Dimension getPreferredSize() {
		return getMaximumSize();
	}
	
	public void setCrystalData(CrystalData crystalData) {
		primitiveVectorTable.setCrystalData(crystalData);
		atomsTable.setCrystalData(crystalData);
		plotPointsTable.setCrystalData(crystalData);
		miscParametersTable.setCrystalData(crystalData);
	}
}

