/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import java.io.Serializable;
import java.util.Arrays;

public class CrystalData implements Cloneable, Serializable{
	public String name;
	public float[] acell;
	public Vector3D[] primitiveVectors;
	public String[] atomNames;
	public int[] zs;
	public int[] atomTypes;
	public Vector3D[] xred;
	public String[] bandPlotPointNames;
	public Vector3D[] bandPlotPoints;
	public double diemac;
	public String psp;
	public float ecut;
	public int maxCycles;
	public int nAtomTypes;
	public int nAtoms;
	public float crystalSize;
	
	public CrystalData(String name) {
		if (name == "Si") {
			this.name = "Si";
			acell = new float[] {10.26311f, 10.26311f, 10.26311f};
			primitiveVectors = new Vector3D[] {
				new Vector3D(0.0f, 0.5f, 0.5f),
				new Vector3D(0.5f, 0.0f, 0.5f), 
				new Vector3D(0.5f, 0.5f, 0.0f)
			};
			atomNames = new String[] {"Si"};
			zs = new int[] {14};
			atomTypes = new int[] {1, 1};
			xred = new Vector3D[] {
				new Vector3D(0.0f, 0.0f, 0.0f), 
				new Vector3D(0.25f, 0.25f, 0.25f)
			};
			bandPlotPointNames = new String[] {"L", "\u0393", "X", "\u0393"};			
			bandPlotPoints = new Vector3D[] {
				new Vector3D(0.5f, 0.0f, 0.0f), 
				new Vector3D(0.0f, 0.0f, 0.0f),
				new Vector3D(0.0f, 0.5f, 0.5f), 
				new Vector3D(1.0f, 1.0f, 1.0f)
			};
			diemac = 12.0;
			psp = "14si.pspnc";
			nAtomTypes = zs.length;
			nAtoms = atomTypes.length;
			ecut = 2.0f;
			maxCycles = 10;
			crystalSize = 1;
		}
		
		if (name == "Ge") {
			this.name = "Ge";
			acell = new float[] {10.681f, 10.681f, 10.681f};
			primitiveVectors = new Vector3D[] {
				new Vector3D(0.0f, 0.5f, 0.5f),
				new Vector3D(0.5f, 0.0f, 0.5f), 
				new Vector3D(0.5f, 0.5f, 0.0f)
			};
			atomNames = new String[] {"Ge"};
			zs = new int[] {32};
			atomTypes = new int[] {1, 1};
			xred = new Vector3D[] {
				new Vector3D(0.0f, 0.0f, 0.0f), 
				new Vector3D(0.25f, 0.25f, 0.25f)
			};
			bandPlotPointNames = new String[] {"L", "\u0393", "X", "\u0393"};			
			bandPlotPoints = new Vector3D[] {
				new Vector3D(0.5f, 0.0f, 0.0f), 
				new Vector3D(0.0f, 0.0f, 0.0f),
				new Vector3D(0.0f, 0.5f, 0.5f), 
				new Vector3D(1.0f, 1.0f, 1.0f)
			};
			diemac = 12.0;
			psp = "32ge.pspnc";
			nAtomTypes = zs.length;
			nAtoms = atomTypes.length;
			ecut = 2.0f;
			maxCycles = 10;
			crystalSize = 1;
		}
	
		if (name == "AlAs") {
			this.name = "AlAs";
			acell = new float[] {10.61f, 10.61f, 10.61f};
			primitiveVectors = new Vector3D[] {
				new Vector3D(0.0f, 0.5f, 0.5f),
				new Vector3D(0.5f, 0.0f, 0.5f), 
				new Vector3D(0.5f, 0.5f, 0.0f)
			};
			atomNames = new String[] {"Al", "As"};
			zs = new int[] {13, 33};
			atomTypes = new int[] {1, 2};
			xred = new Vector3D[] {
				new Vector3D(0.0f, 0.0f, 0.0f), 
				new Vector3D(0.25f, 0.25f, 0.25f)
			};
			bandPlotPointNames = new String[] {"L", "\u0393", "X", "\u0393"};			
			bandPlotPoints = new Vector3D[] {
				new Vector3D(0.5f, 0.0f, 0.0f), 
				new Vector3D(0.0f, 0.0f, 0.0f),
				new Vector3D(0.0f, 0.5f, 0.5f), 
				new Vector3D(1.0f, 1.0f, 1.0f)
			};
			diemac = 9.0;
			psp = "13al.pspnc\n33as.pspnc";
			nAtomTypes = zs.length;
			nAtoms = atomTypes.length;
			ecut = 2.0f;
			maxCycles = 10;
			crystalSize = 1;
		}
		
		if (name == "GaAs") {
			this.name = "GaAs";
			acell = new float[] {10.60f, 10.60f, 10.60f};
			primitiveVectors = new Vector3D[] {
				new Vector3D(0.0f, 0.5f, 0.5f),
				new Vector3D(0.5f, 0.0f, 0.5f), 
				new Vector3D(0.5f, 0.5f, 0.0f)
			};
			atomNames = new String[] {"Ga", "As"};
			zs = new int[] {31, 33};
			atomTypes = new int[] {1, 2};
			xred = new Vector3D[] {
				new Vector3D(0.0f, 0.0f, 0.0f), 
				new Vector3D(0.25f, 0.25f, 0.25f)
			};
			bandPlotPointNames = new String[] {"L", "\u0393", "X", "\u0393"};
			bandPlotPoints = new Vector3D[] {
				new Vector3D(0.5f, 0.0f, 0.0f), 
				new Vector3D(0.0f, 0.0f, 0.0f),
				new Vector3D(0.0f, 0.5f, 0.5f), 
				new Vector3D(1.0f, 1.0f, 1.0f)
			};
			diemac = 10.0;
			psp = "31ga.pspnc\n33as.pspnc";
			nAtomTypes = zs.length;
			nAtoms = atomTypes.length;
			ecut = 2.0f;
			maxCycles = 10;
			crystalSize = 1;
		}
	}
	
	public Vector3D getCartCoordinates(int x, int y, int z, int i) {
		return Vector3D.sum(
			primitiveVectors[0].times(x + xred[i].x),
			primitiveVectors[1].times(y + xred[i].y),
			primitiveVectors[2].times(z + xred[i].z)
			);
	}
	
	public int getAtomIndex(int atomNumber) {
		if (atomNumber >= nAtoms) return -1;
		return atomTypes[atomNumber]-1;
	}
	
	public String getAtomName(int atomNumber) {
		return atomNames[getAtomIndex(atomNumber)];
	}
	
	public int getAtomZ(int atomNumber) {
		return zs[getAtomIndex(atomNumber)];
	}
	
	public float acellAngstrom(int n) {
		return 	0.5291772108f*acell[n];
	}
	
    public boolean equals(Object obj) {
		if (!(obj instanceof CrystalData))
	    	return false;
		CrystalData cd = (CrystalData)obj;
		return cd.name.equals(name)
			&& Arrays.equals(cd.acell, acell)
			&& Arrays.equals(cd.primitiveVectors, primitiveVectors)
			&& Arrays.equals(cd.atomNames, atomNames)
			&& Arrays.equals(cd.atomTypes, atomTypes)
			&& Arrays.equals(cd.xred, xred)
			&& Arrays.equals(cd.bandPlotPointNames, bandPlotPointNames)
			&& Arrays.equals(cd.bandPlotPoints, bandPlotPoints)
			&& cd.diemac == diemac
			&& cd.psp.equals(psp)
			&& cd.ecut == ecut
			&& cd.maxCycles == maxCycles
			&& cd.nAtomTypes == nAtomTypes
			&& cd.nAtoms == nAtoms;
    }
	
	protected Object clone() {
        try {
            CrystalData c = (CrystalData)super.clone();		
			c.acell = (float[])acell.clone();
			c.primitiveVectors = (Vector3D[])primitiveVectors.clone();
			c.atomNames = (String[])atomNames.clone();
			c.zs = (int[])zs.clone();
			c.atomTypes = (int[])atomTypes.clone();
			c.xred = (Vector3D[])xred.clone();
			c.bandPlotPointNames = (String[])bandPlotPointNames.clone();
			c.bandPlotPoints = (Vector3D[])bandPlotPoints.clone();
            return c;				
        } catch (CloneNotSupportedException e) {
            System.out.println(e);
			throw new InternalError();
        }
    } 	
	
	public String toString() {
		return name + "\n"
	 		+ acell + "\n"
		 	+ primitiveVectors + "\n"
		 	+ atomNames + "\n"
		 	+ zs + "\n"
		 	+ atomTypes + "\n"
		 	+ xred + "\n"
			+ bandPlotPointNames + "\n"
			+ bandPlotPoints + "\n"
			+ diemac + "\n"
			+ psp + "\n"
			+ ecut + "\n"
		 	+ maxCycles + "\n"
		 	+ nAtomTypes + "\n"
		 	+ nAtoms;
	}
}
