/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import java.awt.Container;

public class MiscParametersTable extends JTable {

	CrystalData crystalData;
	CrystalPanel crystalPanel;	

	MiscParametersTable() {
		super();
		setModel(new MiscParametersTableModel());
		getTableHeader().setReorderingAllowed(false);
	}
	
	public void setCrystalData(CrystalData crystalData) {
		crystalPanel = (CrystalPanel)((Container)getRootPane().getContentPane().getComponent(0)).getComponent(0);
		this.crystalData = crystalData;
		revalidate();
		repaint();
	}
	
	class MiscParametersTableModel extends AbstractTableModel {
        private String[] columnNames = {" ","value"};
		private String[] parameterNames = {
			"energy cut-off", 
			"max cycles",
			"crystal size"};

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
			return parameterNames.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            if (crystalData == null) return null;
			if (col == 0) 
				return parameterNames[row];
			if (col == 1) {
				if (row == 0) 
					return new Float(crystalData.ecut);
				if (row == 1)
					return new Float(crystalData.maxCycles);
				if (row == 2)
					return new Float(crystalPanel.crystalBounds);
			}
					
			return null;
        }

        public Class getColumnClass(int col) {
			if (col == 0) return String.class;
            return Float.class;
        }

        public boolean isCellEditable(int row, int col) {
			if (col == 0) return false;
			return true;
        }
		
        public void setValueAt(Object value, int row, int col) {
            if (crystalData == null) return;
			if (!(value instanceof Number)) return;
			if (col == 1) {
				if (row == 0)
					crystalData.ecut = ((Float)value).floatValue();
				if (row == 1)
					crystalData.maxCycles = ((Float)value).intValue();
				if (row == 2)
					crystalPanel.crystalBounds = ((Float)value).floatValue();
			}
            fireTableCellUpdated(row, col);
        }
	}
}
