/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.JComponent;

public class PlotPointsTable extends JTable {

	CrystalData crystalData;	

	PlotPointsTable() {
		super();
		setModel(new PlotPointsTableModel());
		getTableHeader().setReorderingAllowed(false);
	}
	
	public void setCrystalData(CrystalData crystalData) {
		this.crystalData = crystalData;
		revalidate();
		repaint();
		((JComponent)getParent()).revalidate();
		((JComponent)getParent()).repaint();
	}

	class PlotPointsTableModel extends AbstractTableModel {
        private String[] columnNames = {" ", "x", "y", "z"};

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
			if (crystalData == null) return 4;
            return crystalData.bandPlotPoints.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            if (crystalData == null) return null;
			if (columnNames[col] == " ") 
				return crystalData.bandPlotPointNames[row];
			if (columnNames[col] == "x")
				return new Float(crystalData.bandPlotPoints[row].x);
			if (columnNames[col] == "y")
				return new Float(crystalData.bandPlotPoints[row].y);
			if (columnNames[col] == "z")
				return new Float(crystalData.bandPlotPoints[row].z);
			return null;
        }

        public Class getColumnClass(int col) {
			if (getColumnName(col) == " ") return String.class;
            return Float.class;
        }

        public boolean isCellEditable(int row, int col) {
			if (getColumnName(col) == " ") return false;
			return true;
        }
		
        public void setValueAt(Object value, int row, int col) {
            if (crystalData == null) return;
			if (!(value instanceof Number)) return;
			if (columnNames[col] == "x")
				crystalData.bandPlotPoints[row].x = ((Float)value).floatValue();
			else if (columnNames[col] == "y")
				crystalData.bandPlotPoints[row].y = ((Float)value).floatValue();
			else if (columnNames[col] == "z")
				crystalData.bandPlotPoints[row].z = ((Float)value).floatValue();
            fireTableCellUpdated(row, col);
        }
	}
}
