/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.SwingConstants;
import java.text.NumberFormat;

public class PrimitiveVectorTable extends JTable {

	CrystalData crystalData;	

	PrimitiveVectorTable() {
		super();
		setModel(new PrimitiveVectorTableModel());
		setDefaultRenderer(Float.class, new FloatRenderer());
		getTableHeader().setReorderingAllowed(false);
	}
	
	public void setCrystalData(CrystalData crystalData) {
		this.crystalData = crystalData;
		revalidate();
		repaint();
	}
	
	public int getTableHeight() {
		return (getRowHeight()+getRowMargin())*getRowCount()+getRowMargin();
	}

	class PrimitiveVectorTableModel extends AbstractTableModel {
        private String[] columnNames = {"a", "x", "y", "z"};

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return 3;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            if (crystalData == null) return null;
			if (columnNames[col] == "a")
				return new Float(crystalData.acell[row]);
			if (columnNames[col] == "x")
				return new Float(crystalData.primitiveVectors[row].x);
			if (columnNames[col] == "y")
				return new Float(crystalData.primitiveVectors[row].y);
			if (columnNames[col] == "z")
				return new Float(crystalData.primitiveVectors[row].z);
			return null;
        }

        public Class getColumnClass(int col) {
            return Float.class;
        }

        public boolean isCellEditable(int row, int col) {
			return true;
        }
		
        public void setValueAt(Object value, int row, int col) {
            if (crystalData == null) return;
			if (!(value instanceof Float)) return;
			if (columnNames[col] == "a")
				crystalData.acell[row] = ((Float)value).floatValue();			
			else if (columnNames[col] == "x")
				crystalData.primitiveVectors[row].x = ((Float)value).floatValue();
			else if (columnNames[col] == "y")
				crystalData.primitiveVectors[row].y = ((Float)value).floatValue();
			else if (columnNames[col] == "z")
				crystalData.primitiveVectors[row].z = ((Float)value).floatValue();
            fireTableCellUpdated(row, col);
        }
	}
	
	static class FloatRenderer extends DefaultTableCellRenderer {
    	NumberFormat formatter;
    	public FloatRenderer() { 
			super(); 
			setHorizontalAlignment(SwingConstants.RIGHT);
		}

    	public void setValue(Object value) {
        	if (formatter==null) {
     	       formatter = NumberFormat.getInstance();
			   formatter.setMaximumFractionDigits(2);
     	   }
      	  setText((value == null) ? "" : formatter.format(value));
   	 	}
	}
}
