/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinit;

import java.io.Serializable;

public class Vector3D implements Cloneable, Serializable {
	public float x,y,z;

	public Vector3D() {
	}

	public Vector3D(float xyz) {
		this.x = xyz;
		this.y = xyz;
		this.z = xyz;
	}

	public Vector3D(float x, float y, float z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public float getX() {
		return x;
	}
	
	public float getY() {
		return y;
	}
	
	public float getZ() {
		return z;
	}

	public float distanceToOrigin() {
		return (float) Math.sqrt(x*x + y*y + z*z);
	}
	
	public Vector3D times(float s) {
		return new Vector3D(s*x, s*y, s*z);
	}
	
	public float dot(Vector3D v) {
		return x*v.x + y*v.y + z*v.z;
	}
	
	public boolean isInBox(Vector3D v1, Vector3D v2) {
		return v1.x <= x && x <= v2.x
			&& v1.y <= y && y <= v2.y
			&& v1.z <= z && z <= v2.z;
			
	}
	
	public float norm() {
		return (float)Math.sqrt(dot(this));
	}
	
	public float angle(Vector3D v) {
		return 180f*(float)Math.acos(this.dot(v)/(this.norm()*v.norm()))/(float)Math.PI;
	}
	
	public static Vector3D sum(Vector3D v1, Vector3D v2) {
		return new Vector3D(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
	}
	
	public static Vector3D sum(Vector3D v1, Vector3D v2, Vector3D v3) {
		return new Vector3D(v1.x + v2.x + v3.x, v1.y + v2.y + v3.y, v1.z + v2.z + v3.z);
	}
	
	public String toString() {
		return "" + x + " " + y + " " + z;
	}

	protected Object clone() {
        try {	
            return super.clone();				
        } catch (CloneNotSupportedException e) {
            System.out.println(e);
			throw new InternalError();
        }
    }
	
    public boolean equals(Object obj) {
		if (!(obj instanceof Vector3D))
	    	return false;
		Vector3D v3d = (Vector3D)obj;
		return v3d.x == x && v3d.y == y && v3d.z == z;
	}
}

