/* This file is part of Jabinit, copyright (C) 2005 Simon Rochester and John C. Morrison
*
* Jabinit is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* Jabinit is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package edu.louisville.physics.jcm.jabinitserver;

import java.net.Socket;
import java.io.BufferedWriter;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Arrays;


public class JabinitConnection extends Thread {
	private Socket clientSocket = null;
	private File dir, inputFile, outputFile, batFile, filesFile;
	private String dirString;

	public JabinitConnection(Socket socket) {
		super("JabinitConnection");
		this.clientSocket = socket;
		
		//directory for aux files
		try {
			dirString = clientSocket.getRemoteSocketAddress().toString().replaceAll(":","-").substring(1);
			dir = File.createTempFile(dirString,"");
			dir.delete();
			dir.mkdir();
			
			batFile = new File(dir, "jabinit.sh");
			BufferedWriter toFile = new BufferedWriter(new FileWriter(batFile));
			toFile.write("#!/bin/bash\n" + "bin/abinis < " + dir.getPath() + "/jabinit.files >& " + dir.getPath() + "/jabinit.log");
			toFile.close();
			inputFile = new File(dir, "jabinit.in");
			outputFile = new File(dir, "jabinit.out");
			filesFile = new File(dir, "jabinit.files");
			
			System.out.println("Connection to " + this.clientSocket.getInetAddress().toString() + " established on: " + new Date());
		} catch (IOException e) { 
			System.out.println(e);
			System.exit(1);
		}
	}

	public void run() { 
		PrintWriter toClient = null;
		BufferedReader fromClient = null;
		Process process = null;

		try	{
			for (;;) {
				//create input and output streams for communicating with the client
				toClient = new PrintWriter(clientSocket.getOutputStream(), true);
				fromClient = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));

				String line;
				troll:
				for (;;) {
					//read input data from the client and write it to file
					
					//get # of atom types
					int ntypat = new Integer(fromClient.readLine()).intValue();
					
					//create files file
					if (filesFile.exists()) filesFile.delete();
					BufferedWriter toFile = new BufferedWriter(new FileWriter(filesFile));
					toFile.write(
						inputFile.getPath() +          "\n"
						+ outputFile.getPath() +       "\n"
						+ dir.getPath() + "/jabiniti        \n"
						+ dir.getPath() + "/jabinito        \n"
						+ dir.getPath() + "/jabinit         \n"
					);

					//get and write psp files
					for (int i=0; i<ntypat; i++) {
						toFile.write("psps/" + fromClient.readLine() + "\n");
					}
					toFile.close();
					
					//get and write input file data
					if (inputFile.exists()) inputFile.delete();
					toFile = new BufferedWriter(new FileWriter(inputFile));
					while (!(line = fromClient.readLine()).equals("END")) {
						if (line.equals("EXIT")) break troll;
						toFile.write(line);
						toFile.newLine();
					}
					toFile.close();

					//delete old output file
					if (outputFile.exists()) outputFile.delete();

					//run abinit
					Runtime runtime = Runtime.getRuntime();
					String batPath = batFile.getPath();
					
					process = runtime.exec("chmod 755 " + batPath);    
					process.waitFor();
					process = runtime.exec(batPath);
					System.out.println("Started process: " + process);
					process.waitFor();
					System.out.println("Process finished: " + process);
					process.destroy();

					//read data from the output file and feed it to the client
					BufferedReader fromFile = new BufferedReader(new FileReader(outputFile));
					while ((line = fromFile.readLine()) != null) {
		//				System.out.println(line);
						toClient.println(line);
					}
					fromFile.close();
					toClient.println("END");
					
				}

				if (line.equals("EXIT")) break;
			}
		} catch (IOException e)	{
			System.err.println(e.toString());
		} catch (InterruptedException e)	{
			System.err.println(e.toString());
		} catch (NumberFormatException e) {
		} finally {
			try	{
				if (dir != null) {
					String[] filelist = dir.list();
					for (int i=0; i<filelist.length; i++) {
						File file = new File(dir, filelist[i]);
						file.delete();
					}
				}
				dir.delete();
				if (process != null) process.destroy();
				if (toClient != null) toClient.close();
				if (fromClient != null) fromClient.close();
				if (clientSocket != null) clientSocket.close();
			} catch (IOException e)	{
				System.err.println(e.toString());
			}
		}
	}
}
