/* $RCSfile: VectorsRenderer.java,v $
 * $Author: migueljmol $
 * $Date: 2004/11/18 20:52:31 $
 * $Revision: 1.1 $
 *
 * Copyright (C) 2002-2004  The Jmol Development Team
 *
 * Contact: jmol-developers@lists.sf.net
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

package org.jmol.viewer;

import org.jmol.g3d.*;
import javax.vecmath.*;

class VectorsRenderer extends ShapeRenderer {

  void render() {
    if (! frame.hasVibrationVectors)
      return;
    Atom[] atoms = frame.atoms;
    Vectors vectors = (Vectors)shape;
    short[] mads = vectors.mads;
    if (mads == null)
      return;
    short[] colixes = vectors.colixes;
    int displayModelIndex = this.displayModelIndex;
    for (int i = frame.atomCount; --i >= 0; ) {
      Atom atom = atoms[i];
      if (mads[i] == 0 ||
          (displayModelIndex >= 0 && atom.modelIndex != displayModelIndex))
        continue;
      Vector3f vibrationVector = atom.getVibrationVector();
      if (vibrationVector == null)
        continue;
      if (transform(mads[i], atom, vibrationVector))
        renderVector(colixes[i], atom);
    }
  }

  final Point3f pointVectorEnd = new Point3f();
  final Point3f pointArrowHead = new Point3f();
  final Point3i screenVectorEnd = new Point3i();
  final Point3i screenArrowHead = new Point3i();
  final Vector3f vibrationVectorScaled = new Vector3f();
  int diameter;
  float headWidthAngstroms;
  int headWidthPixels;

  final static float arrowHeadBase = 0.8f;

  boolean transform(short mad, Atom atom, Vector3f vibrationVector) {
    if (atom.madAtom == JmolConstants.MAR_DELETED)
      return false;

    // to have the vectors stay in the the same spot
    /*
    float vectorScale = viewer.getVectorScale();
    pointVectorEnd.scaleAdd(vectorScale, atom.vibrationVector, atom.point3f);
    viewer.transformPoint(pointVectorEnd, screenVectorEnd);
    diameter = (mad <= 20)
      ? mad
      : viewer.scaleToScreen(screenVectorEnd.z, mad);
    pointArrowHead.scaleAdd(vectorScale * arrowHeadBase,
                            atom.vibrationVector, atom.point3f);
    viewer.transformPoint(pointArrowHead, screenArrowHead);
    headWidthPixels = diameter * 3 / 2;
    if (headWidthPixels < diameter + 2)
      headWidthPixels = diameter + 2;
    return true;
    */

    // to have the vectors move when vibration is turned on
    float vectorScale = viewer.getVectorScale();
    pointVectorEnd.scaleAdd(vectorScale, vibrationVector, atom.point3f);
    viewer.transformPoint(pointVectorEnd, vibrationVector,
                          screenVectorEnd);
    diameter = (mad <= 20)
      ? mad
      : viewer.scaleToScreen(screenVectorEnd.z, mad);
    pointArrowHead.scaleAdd(vectorScale * arrowHeadBase,
                            vibrationVector, atom.point3f);
    viewer.transformPoint(pointArrowHead, vibrationVector,
                          screenArrowHead);
    headWidthPixels = diameter * 3 / 2;
    if (headWidthPixels < diameter + 2)
      headWidthPixels = diameter + 2;
    return true;
  }
  
  void renderVector(short colix, Atom atom) {
    if (colix == 0)
      colix = atom.colixAtom;
    g3d.fillCylinder(colix, Graphics3D.ENDCAPS_OPEN, diameter,
                 atom.getScreenX(), atom.getScreenY(), atom.getScreenZ(),
                 screenArrowHead.x, screenArrowHead.y, screenArrowHead.z);
    g3d.fillCone(colix, Graphics3D.ENDCAPS_NONE, headWidthPixels,
                 screenArrowHead, screenVectorEnd);
  }
}
